require 'stripe2qb/options_reading'
require 'yaml'
require 'erb'

module Stripe2QB
  class ConfigurationError < StandardError; end

  class Configuration
    include OptionsReading

    attr_reader :quickbooks_api
    attr_reader :stripe_api

    def initialize(options)
      if options.is_a?(String)
        file = File.open(options)
        options = YAML.load(ERB.new(file.read).result)
      end

      quickbooks_api_options = get_required_from_options('quickbooks_api', options)
      @quickbooks_api = QuickbooksApi.new(quickbooks_api_options)

      stripe_api_options = get_required_from_options('stripe_api', options)
      @stripe_api = StripeApi.new(stripe_api_options)
    end

    def process(start_date, end_date = nil)
      transfers = stripe_api.get_transfers(start_date, end_date || Date.today)
      transfers.each do |transfer|
        Converters::TransferToDeposit.new(transfer, self).find_or_create
      end
    end
  end
end
