module Stripe2QB
  module Converters
    class Base
      attr_reader :quickbooks_api
      attr_reader :stripe_api

      def initialize(configuration)
        @quickbooks_api = configuration.quickbooks_api
        @stripe_api = configuration.stripe_api
      end

      def find
        raise 'not implemented'
      end

      def exists?
        !find.nil?
      end

      def create!
        raise 'not implemented'
      end

      def find_or_create
        find || create!
      end

      def delete!
        raise 'not implemented'
      end

      protected

      def format_date(value)
        Time.at(value).in_time_zone('UTC').to_date.to_s
      end

      def format_amount(value)
        (value / 100.0).round(2)
      end
    end
  end
end
