require 'stripe2qb/quickbooks_api/access_token'
require 'active_support/inflector'
require 'quickbooks-ruby'

module Stripe2QB
  class QuickbooksApi
    include OptionsReading
    include ActiveSupport::Inflector

    attr_reader :realm_id
    attr_reader :access_token
    attr_reader :receipt_customer
    attr_reader :receipt_item
    attr_reader :receipt_payment_method
    attr_reader :receipt_account
    attr_reader :deposit_account
    attr_reader :deposit_fees_vendor
    attr_reader :deposit_fees_account

    def initialize(options)
      set_attribute_from_options('realm_id', options)
      @access_token = AccessToken.new(options)
      @receipt_customer = find_by_name_or_id(:customer, 'receipt_customer', options, :display_name)
      @receipt_item = find_by_name_or_id(:item, 'receipt_item', options)
      @receipt_payment_method = find_by_name_or_id(:payment_method, 'receipt_payment_method', options)
      @receipt_account = find_by_name_or_id(:account, 'receipt_account', options)
      @deposit_account = find_by_name_or_id(:account, 'deposit_account', options)
      @deposit_fees_vendor = find_by_name_or_id(:vendor, 'deposit_fees_vendor', options, :display_name)
      @deposit_fees_account = find_by_name_or_id(:account, 'deposit_fees_account', options)
    end

    def renew_access_token
      result = access_token_service.renew

      { token: result.token, secret: result.secret, created_on: Date.today }
    end

    def method_missing(symbol, *args)
      if symbol.to_s =~ /^(.+)_service$/
        get_service($1.camelize)
      else
        super
      end
    end

    private

    def find_by_name_or_id(type, key, options, name_attr = :name)
      value = get_required_from_options(key, options)
      service = send("#{type}_service")

      unless object = value.is_a?(String) ? service.find_by(name_attr, value).first : service.find_by(:id, value).first
        msg = value.is_a?(String) ? "#{name_attr} '#{value}'" : "ID #{value}"
        raise ConfigurationError.new("no #{type} for #{msg}")
      end

      object
    end

    def get_service(name)
      service = "Quickbooks::Service::#{name}".constantize.new
      service.access_token = access_token.oauth
      service.company_id = realm_id

      service
    end
  end
end
