require 'stripe'

module Stripe2QB
  class StripeApi
    include OptionsReading

    attr_reader :api_key

    def initialize(options)
      Stripe.api_key = set_attribute_from_options('api_key', options)
    end

    def get_transfers(start_date, end_date = nil)
      Stripe::Transfer.list(date: date_range_criteria(start_date, end_date), status: 'paid', limit: 100)
    end

    def get_transfer_charges(transfer_id)
      Stripe::BalanceTransaction.all(transfer: transfer_id, type: 'charge', limit: 100)
    end

    def get_transfer_refunds(transfer_id)
      Stripe::BalanceTransaction.all(transfer: transfer_id, type: 'refund', limit: 100)
    end

    private

    def date_range_criteria(start_date, end_date = nil)
      start_time = to_time(start_date)
      end_time = to_time(end_date || Date.parse(start_date) + 1.day)

      { gte: start_time, lt: end_time }
    end

    def to_time(date)
      date = Date.parse(date) if date.is_a?(String)

      Time.parse("#{date} 0:00 UTC").to_i
    end
  end
end
